{=== 2D Ising model show (see also ISING.PAS for 3D) ===}

uses CRT,getdata,graph;

type myreal=single;

var T,h:myreal;
const Lmax=248;

type cfg=array[-1..Lmax,-1..Lmax] of shortint;
var s:cfg;

var de:array[-1..1,-6..6] of myreal;
var i,j,k,E,m,L,L1,nint,dLx,dLy:integer;
var ss:shortint;
var ht,mt,st,dt:word;

var sx,sy:myreal;
var graphdriver,graphmode:integer;
var C1,C2,aspx,aspy,dxs,dys:word;

type bctype=(box,slit,cyclic,iface);
var bc:bctype;
var isiface:boolean;

var p:myreal;

var atx,aty:integer;
procedure keyhelp(key,msg:string);
begin
setcolor(yellow);
outtextxy(atx,aty,key);
setcolor(lightcyan);
outtextxy(atx,aty+12,msg);
aty:=aty+40;
end;

PROCEDURE Init;
var ss,i:integer;
var st:string[15];
BEGIN

L1:=L-1;
getaspectratio(aspx,aspy);
sy:=getmaxy/(L+2);
sx:=sy*aspy/aspx;
dxs:=trunc(sx);
dys:=trunc(sy);

case bc of
  box: begin dLx:=1; dLy:=1; end;
  slit: begin dLx:=0; dLy:=1; end;
  cyclic: begin dLx:=0; dLy:=0; end;
  end;

for ss:=-1 to 1 do
  for i:=-6 to 6 do de[ss][i]:=exp((-2*i+2*h)*ss/T);

setcolor(white);
settextstyle(0,0,2);
setfillstyle(solidfill,black);
bar(getmaxx*10 div 13,0,getmaxx,64);
str(T:4:3,st);
outtextxy(getmaxx*10 div 13,1,'T='+st);
str(h:4:2,st);
outtextxy(getmaxx*10 div 13,33,'h='+st);

atx:=getmaxx*10 div 13;
aty:=80;
settextstyle(0,0,1);
keyhelp('t T - +','change temperature');
keyhelp('h H','change magn. field');
keyhelp('c','critical point');
if bc<>cyclic then keyhelp('i','invert boundary c.')
else BEGIN
  keyhelp('r','renorm. group 3*3');
  keyhelp('R','expand 1->3*3'); END;
keyhelp('f','flip all spins');
keyhelp('s','flip seed (small)');
keyhelp('S','flip seed (large)');
keyhelp('SPACE','freeze/go');
keyhelp('ESC','exit');
end;


PROCEDURE drawspin(i,j,ss:integer);
var x0,y0:integer;
BEGIN
if ss=1 then setfillstyle(solidfill,C1) else setfillstyle(solidfill,C2);
x0:=round(sx*succ(i)); y0:=round(sy*succ(j));
bar(x0,y0,x0+dxs,y0+dys);
END;

PROCEDURE DrawSpins;
BEGIN
for i:=-dLx to L1+dLx do for j:=-dLy to L1+dLy do drawspin(i,j,s[i,j]);
END;

procedure getspin(s:string);
BEGIN
write(s,' spins (+/-/Random/Cancel) ? ');
REPEAT UNTIL UpKey in ['+','-','R','C'];
writeln(LastChar);
if LastChar='C' then if Yes('really cancel') then halt;
{ ...lastchar passed to function spin }
END;

function spin:shortint;
begin
case lastchar of
  '+': spin:=+1;
  '-': spin:=-1;
  'R': if random<0.5 then spin:=+1 else spin:=-1;
  end;
end;

PROCEDURE flipspin(i,j:integer);
BEGIN
s[i,j]:=-s[i,j];
drawspin(i,j,s[i,j]);
END;

PROCEDURE setborders;
BEGIN
if bc>=slit then
  for i:=0 to L1 do begin s[-1,i]:=s[L1,i]; s[L,i]:=s[0,i] end;
if bc=cyclic then
  for i:=0 to L1 do begin s[i,-1]:=s[i,L1]; s[i,L]:=s[i,0] end;
END;

PROCEDURE RG;
var ss,ii,jj:integer;
BEGIN
if bc<>cyclic then exit;
REPEAT
  if L>=9 then BEGIN
    L:=L div 3;
    Init;
    ClearViewPort;
    for i:=0 to L1 do for j:=0 to L1 do BEGIN
      ss:=0;
      for ii:=i*3 to i*3+2 do for jj:=j*3 to j*3+2 do ss:=ss+s[ii,jj];
      if ss>0 then s[i,j]:=1 else s[i,j]:=-1;
      END; END;
  setborders;
  DrawSpins;
  aty:=80;
  keyhelp('r','once more');
  keyhelp('any key','continue')
  UNTIL readkey<>'r';
setfillstyle(0,0);
bar(atx,80,getmaxx,aty);
END;

PROCEDURE expandRG;
var ss,ii,jj:integer;
BEGIN
if bc<>cyclic then exit;
REPEAT
  if 3*L<=Lmax then BEGIN
    L:=3*L;
    Init;
    ClearViewPort;
    for i:=L1 downto 0 do for j:=L1 downto 0 do
      s[i,j]:=s[i div 3,j div 3]; END;
  setborders;
  DrawSpins;
  aty:=80;
  keyhelp('R','once more');
  keyhelp('any key','continue')
  UNTIL readkey<>'R';
setfillstyle(0,0);
bar(atx,80,getmaxx,aty);
END;

{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

BEGIN

Randomize;

textcolor(white);
writeln;
writeln('Metropolis Monte Carlo simulation');
writeln('of Ising ferromagnet on a square lattice');
writeln('(c) J. Kolafa 1993-7');
writeln;
writeln('Interaction Hamiltonian:');
writeln('  H = - SUM  s_i s_j + h SUM s_i');
writeln('       <i,j>              i');
writeln('where spins s_i={-1,1}, h is magnetic field,');
writeln('the 1st sum is over all edges and the 2nd sum over all vertices');
writeln;

textcolor(yellow);
writeln('select boundary conditions:');
bc:=bctype(GetInteger('  0=walls  1=periodic in x  2=periodic  3=horizontal interface',2));
if not (bc in [box,slit,cyclic,iface]) then halt;

L:=78; if bc=cyclic then L:=81;
L:=GetInteger('simulation square edge (max. 248)', L);
if L>Lmax then begin
  writeln('sorry, maximum is ',Lmax);
  halt; end;
if L<3 then begin
  writeln('too small lattice');
  halt; end;

L1:=L-1;
isiface:=bc=iface;
if isiface then bc:=box;
T:=GetReal('initial temperature T', 1.666803);
if T<0.25 then T:=0.25;
if T>1e5 then T:=1e5;

h:=GetReal('inital magnetic field h',0);
if h>4 then h:=4;
if h<-4 then h:=-4;

{ for the sake of expandRG }
for i:=0 to Lmax do for j:=0 to Lmax do
  if random<p then s[i,j]:=1 else s[i,j]:=-1;

if isiface then
  for j:=-1 to L div 2-1 do  for i:=-1 to L do begin
    s[i,j]:=1; s[i,L1-j]:=-1; end

else begin

  { init cfg }
  p:=GetReal('initial probability of spin +1',0.5);
  for i:=0 to L1 do for j:=0 to L1 do
    if random<p then s[i,j]:=1 else s[i,j]:=-1;

  setborders;

  if bc<=slit then begin
    getspin('top row');
    for i:=-1 to L do s[i,-1]:=spin;
    getspin('bottom row');
    for i:=-1 to L do s[i,L]:=spin; end;

  if bc=box then begin
    getspin('left column');
    for i:=-1 to L do s[-1,i]:=spin;
    getspin('right column');
    for i:=-1 to L do s[L,i]:=spin; end;
  end;

C1:=GetInteger('color of spin +1 (0=black 1=blue)',blue);
C2:=GetInteger('color of spin -1 (15=white 14=yellow)',yellow);

graphdriver:=detect;
initgraph(graphdriver,graphmode,'');

Init;
DrawSpins;

REPEAT

{ this is a bit optimized }

case bc of

box: for nint:=1 to 32 do begin
  i:=random(L); j:=random(L);
  ss:=s[i,j];
  E:=( s[succ(i),j] + s[pred(i),j]
     + s[i,succ(j)] + s[i,pred(j)] );
  if random<de[ss][E] then BEGIN
    ss:=-ss;
    s[i,j]:=ss;
    drawspin(i,j,ss); END;
  end;

slit: for nint:=1 to 32 do begin
  i:=random(L); j:=random(L);
  ss:=s[i,j];
  E:=( s[succ(i),j] + s[pred(i),j]
     + s[i,succ(j)] + s[i,pred(j)] );
  if random<de[ss][E] then BEGIN
    ss:=-ss;
    s[i,j]:=ss;
    if i=0 then s[L,j]:=ss;
    if i=L1 then s[-1,j]:=ss;
    drawspin(i,j,ss); END;
  end;

cyclic: for nint:=1 to 32 do begin
  i:=random(L); j:=random(L);
  ss:=s[i,j];
  E:=( s[succ(i),j] + s[pred(i),j]
     + s[i,succ(j)] + s[i,pred(j)] );
  if random<de[ss][E] then BEGIN
    ss:=-ss;
    s[i,j]:=ss;
    if i=0 then s[L,j]:=ss;
    if i=L1 then s[-1,j]:=ss;
    if j=0 then s[i,L]:=ss;
    if j=L1 then s[i,-1]:=ss;
    drawspin(i,j,ss); END;
  end;
end;

if keypressed then begin
  case readkey of
    '+','T': if T<1e5 then T:=T*1.0473972;
    '-','t': if T>0.25 then T:=T/1.0473972;
    'h': if h>-2.499 then h:=h-0.01;
    'H': if h<2.499 then h:=h+0.01;
    'c','C': begin T:=1+sqrt(2); h:=0; end;
    'i','I': begin
      if dlx<>0 then for j:=-1 to L do begin
        flipspin(-1,j);
        flipspin(L,j); end;
      if dly<>0 then for i:=0 to L1 do begin
        flipspin(i,-1);
        flipspin(i,L); end;
      end;
    ' ': if readkey=^[ then begin closegraph; halt; end;
    'S':
      for i:=L div 4 to 3*L div 4 do
        for j:=L div 4 to 3*L div 4 do
          flipspin(i,j);
    's':
      for i:=2*L div 5 to 3*L div 5 do
        for j:=2*L div 5 to 3*L div 5 do
          flipspin(i,j);
    'f':
      begin
      for i:=0 to L1 do for j:=0 to L1 do flipspin(i,j);
      setborders;
      end;

    'r': RG;
    'R': expandRG;

    ^[: begin closegraph; halt; end;
    end;
  if abs(h)<0.005 then h:=0;
  Init; end;

until false;
END.
